/**
 * Copyright 2005 The Apache Software Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.nutch.admin.crawldb;

import java.io.File;
import java.io.IOException;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import java.util.StringTokenizer;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.io.MapFile;
import org.apache.hadoop.io.UTF8;
import org.apache.hadoop.io.Writable;
import org.apache.hadoop.mapred.MapFileOutputFormat;
import org.apache.hadoop.mapred.lib.HashPartitioner;
import org.apache.nutch.crawl.CrawlDatum;
import org.apache.nutch.crawl.MapWritable;

public class CrawlDbStatistic {

  /**
   * @param crawlDb
   * @param url
   * @param config
   * @return the statistic for an url
   * @throws IOException
   */
  public List readUrl(File crawlDb, String url, Configuration config)
      throws IOException {
    List list = new LinkedList();
    FileSystem fs = FileSystem.get(config);
    UTF8 key = new UTF8(url);
    CrawlDatum val = new CrawlDatum();
    if (fs.exists(crawlDb)) {
      MapFile.Reader[] readers =
          MapFileOutputFormat.getReaders(
              fs, 
              new File(crawlDb, CrawlDatum.DB_DIR_NAME), 
              config
              );
      
      Writable res =
          MapFileOutputFormat.getEntry(
                  readers, 
                  new HashPartitioner(), 
                  key, 
                  val
                  );
      
      StringBuffer buffer = new StringBuffer();
      buffer.append(url);
      buffer.append("\n");
      if (res != null) {
        buffer.append(val);
        buffer.append("meta datas: ");
        MapWritable metaData = val.getMetaData();
        if(metaData != null){
          Set keySet = metaData.keySet();
          Iterator iterator = keySet.iterator();
          while (iterator.hasNext()) {
            Writable keyWritable = (Writable) iterator.next();
           
            MapWritable otherMapWritable =
                (MapWritable) metaData.get(keyWritable);
            
            Set keySet2 = otherMapWritable.keySet();
            Iterator iterator2 = keySet2.iterator();
            buffer.append(keyWritable.toString() + ":");
            while (iterator2.hasNext()) {
              LongWritable longWritable = (LongWritable) iterator2.next();
              UTF8 metadataValue = (UTF8) otherMapWritable.get(longWritable);
              buffer.append(metadataValue.toString());
              buffer.append("   ");
            }
  
          }
        }
      } else {
        buffer.append("not found");
      }
      StringTokenizer tokenizer = new StringTokenizer(buffer.toString(), "\n");

      List tmpList = new LinkedList();
      while (tokenizer.hasMoreTokens()) {
        String token = tokenizer.nextToken();
        tmpList.add(token);
      }
      list.add(tmpList.toArray(new String[tmpList.size()]));
    }
    return list;
  }
}
