/**
 * Copyright 2005 The Apache Software Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.nutch.admin.crawldb;

import java.io.File;
import java.io.IOException;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import java.util.StringTokenizer;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.io.MapFile;
import org.apache.hadoop.io.UTF8;
import org.apache.hadoop.io.Writable;
import org.apache.hadoop.io.WritableComparable;
import org.apache.hadoop.io.MapFile.Reader;
import org.apache.hadoop.mapred.MapFileOutputFormat;
import org.apache.nutch.crawl.CrawlDatum;
import org.apache.nutch.crawl.MapWritable;

public class UrlWithStatus {

  private static UrlWithStatus instance;

  private Set fReaderSet;

  private UrlWithStatus(File crawlDb, Configuration configuration)
      throws IOException {
    this.fReaderSet = new HashSet();
    FileSystem fs = FileSystem.get(configuration);
    if (fs.exists(crawlDb)) {
      MapFile.Reader[] readers =
          MapFileOutputFormat.getReaders(
              fs, 
              new File(crawlDb, CrawlDatum.DB_DIR_NAME), 
              configuration
              );
      for (int i = 0; i < readers.length; i++) {
        Reader reader = readers[i];
        this.fReaderSet.add(reader);
      }
    }
  }

  public static UrlWithStatus getInstance(
      File crawlDb, Configuration configuration, boolean reset)
      throws IOException {
   
    if (instance == null || reset) {
      instance = new UrlWithStatus(crawlDb, configuration);
    }

    return instance;
  }

  public List readUrl(byte status, int maxUrls) throws IOException,
      InstantiationException, IllegalAccessException {
    
    List list = new LinkedList();
    Iterator iterator = this.fReaderSet.iterator();
    while (iterator.hasNext()) {
      Reader reader = (Reader) iterator.next();
      if (reader != null) {
        WritableComparable key =
            (WritableComparable) reader.getKeyClass().newInstance();
        Writable value = (Writable) reader.getValueClass().newInstance();
        int counter = 0;
        while (counter < maxUrls && reader.next(key, value)) {
          CrawlDatum crawlDatum = (CrawlDatum) value;
          if (crawlDatum.getStatus() == status) {
            counter++;
            StringBuffer buffer = new StringBuffer();
            buffer.append(key.toString());
            buffer.append("\n");
            buffer.append(crawlDatum.toString());
            MapWritable metaData = crawlDatum.getMetaData();
            if (metaData != null && !metaData.isEmpty()) {
              Set keySet = metaData.keySet();
              Iterator metaIter = keySet.iterator();
              while (metaIter.hasNext()) {
                Writable keyWritable = (Writable) metaIter.next();
                MapWritable otherMapWritable =
                    (MapWritable) metaData.get(keyWritable);
                Set keySet2 = otherMapWritable.keySet();
                Iterator iterator2 = keySet2.iterator();
                buffer.append(keyWritable.toString() + ":");
                while (iterator2.hasNext()) {
                  LongWritable longWritable = (LongWritable) iterator2.next();
                  UTF8 metadataValue =
                      (UTF8) otherMapWritable.get(longWritable);
                  buffer.append(metadataValue.toString());
                  buffer.append("   ");
                }
              }
            }
            StringTokenizer tokenizer =
                new StringTokenizer(buffer.toString(), "\n");

            List tmpList = new LinkedList();
            while (tokenizer.hasMoreTokens()) {
              String token = tokenizer.nextToken();
              tmpList.add(token);
            }
            list.add(tmpList.toArray(new String[tmpList.size()]));
          }
        }
      }
    }
    return list;
  }

}
