/**
 * Copyright 2005 The Apache Software Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.nutch.admin.management;

import java.io.File;
import java.io.IOException;
import java.util.logging.Logger;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FileSystem;
import org.apache.nutch.admin.TaskThread;
import org.apache.nutch.crawl.LinkDb;

public class LinkdbThread extends TaskThread {

  private static final Logger LOG = 
    Logger.getLogger(LinkdbThread.class.getName());

  private LinkDb fLinkdb;

  private File[] fSegments;

  private File fLinkdbFile;
 
  public LinkdbThread(Configuration configuration, File linkdb, File[] segments) {
    super(configuration);
    this.fLinkdb = new LinkDb(configuration);
    this.fLinkdbFile = linkdb;
    this.fSegments = segments;
  }

  public void run() {
    FileSystem fileSystem = null;
    try {
      this.fMessage = "linkdb.running";
      fileSystem = FileSystem.get(this.fConfiguration);
      File running = new File(this.fLinkdbFile, "linkdb.running");
      // create lock files in segments
      for (int i = 0; i < this.fSegments.length; i++) {
        File file = this.fSegments[i];
        fileSystem.createNewFile(new File(file, "linkdb.running"));
      }
      // create lockfile in linkdb
      fileSystem.createNewFile(running);

      this.fLinkdb.invert(this.fLinkdbFile, this.fSegments);
      for (int i = 0; i < this.fSegments.length; i++) {
        File file = this.fSegments[i];
        fileSystem.createNewFile(new File(file, "invert.done"));
      }
    } catch (IOException e) {
      LOG.warning(e.toString());
      this.fMessage = e.toString();
    } finally {
      try {
        if (fileSystem != null) {
          // delete lock files and create done files
          for (int i = 0; i < this.fSegments.length; i++) {
            File file = this.fSegments[i];
            fileSystem.delete(new File(file, "linkdb.running"));
          }
          // delete lock file from linkdb
          fileSystem.delete(new File(this.fLinkdbFile, "linkdb.running"));
        }
      } catch (IOException e) {
        LOG.warning(e.toString());
      }
    }
  }
}
