/**
 * Copyright 2005 The Apache Software Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.nutch.admin.management;

import java.io.File;
import java.io.IOException;
import java.util.logging.Logger;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FileSystem;
import org.apache.nutch.admin.TaskThread;
import org.apache.nutch.parse.ParseSegment;

public class ParseThread extends TaskThread {

  private static final Logger LOG = 
    Logger.getLogger(ParseThread.class.getName());

  private ParseSegment fParser;

  private File fSegment;
  
  public ParseThread(Configuration configuration, File segment) {
    super(configuration);
    this.fParser = new ParseSegment(configuration);
    this.fSegment = segment;
  }

  public void run() {
    FileSystem fileSystem = null;
    File running = null;
    try {
      this.fMessage = "parse.running";
      fileSystem = FileSystem.get(this.fConfiguration);
      running = new File(this.fSegment, "parse.running");
      fileSystem.createNewFile(running);
      this.fParser.parse(this.fSegment);
      fileSystem.createNewFile(new File(this.fSegment, "parse.done"));
    } catch (IOException e) {
      LOG.warning(e.toString());
      this.fMessage = e.toString();
    } finally {
      try {
        if (fileSystem != null) {
          fileSystem.delete(running);
        }
      } catch (IOException e) {
        LOG.warning(e.toString());
      }
    }
  }
}
